from typing import Any

from sqlalchemy.ext.asyncio import AsyncSession
from sqlmodel import SQLModel
from ..schemas.params import QueryParams


class CoreService:

    def __init__(self, db: AsyncSession, template: SQLModel | Any):
        self.repository = template.repository(db, template)
        self.template = template
        self.db = db

    async def create_one_record(self, schema_create: SQLModel | Any):
        db_record = await self.repository.create_one_record(schema_create)
        return db_record

    async def update_one_record(self, record_id: int, schema_update: SQLModel | Any):
        db_record = await self.repository.update_one_record(record_id, schema_update)
        return db_record

    async def delete_one_record(self, record_id: int):
        db_record = await self.repository.delete_one_record(record_id)
        return db_record

    async def get_one(self, record_id: int):
        result = await self.repository.get_one(self.db, record_id)
        return result

    async def get_list(self, param: QueryParams):
        result = await self.repository.get_list(self.db, param)
        return result

    async def get_list_with_count(self, param: QueryParams):
        result = await self.repository.get_list_with_count(self.db, param)
        return result



from typing import Union, Any

from fastapi import HTTPException
from pydantic import BaseModel
from sqlmodel import SQLModel
from sqlalchemy.ext.asyncio import AsyncSession


class CoreUpdate:
    async def get_pk_name(self, model_class: SQLModel):
        return model_class.__table__.primary_key.columns.values()[0].name if model_class else None

    async def update_one_record(self, db: AsyncSession, record_id: Union[int, str], model_class: SQLModel | Any,
                                schema_create: BaseModel | Any):
        pk_name = await self.get_pk_name(model_class)
        update_pk_value = schema_create.model_dump().get(pk_name)

        if update_pk_value != record_id:
            raise HTTPException(status_code=400,
                                detail=f"Cannot change record id form {record_id} to {update_pk_value}")

        db_model = await db.get(model_class, record_id)
        if not db_model:
            raise HTTPException(status_code=404, detail="Record not found")

        data = schema_create.model_dump(exclude_unset=True)
        db_model.sqlmodel_update(data)
        db.add(db_model)
        await db.flush()
        return db_model



from cnb.utils.sensitive_data import sensitive_keys


def mask_sensitive_data(data: dict) -> dict:
    """
    Masks sensitive fields in a dictionary.

    Args:
        data (dict): The data dictionary to mask (headers or body).
    Returns:
        dict: A new dictionary with sensitive fields masked.
    """
    masked_data = data.copy()
    for key in sensitive_keys:
        if key in masked_data:
            data = str(masked_data[key])
            if len(data) <= 5:
                masked_data[key] = f"***********{data[-1:]}"  # Replace sensitive value with mask
            else:
                masked_data[key] = f"***********{data[-3:]}"
    return masked_data



import asyncio
import logging
from fastapi import Depends
from sqlalchemy.exc import InterfaceError
from sqlalchemy import URL, make_url
from sqlalchemy.ext.asyncio import AsyncEngine
from sqlmodel import create_engine
from sqlmodel.ext.asyncio.session import AsyncSession
from sqlalchemy.orm import sessionmaker

from cnb.db.postgres.pg_settings import pg_settings

engine = None


async def get_postgres_engine():
    try:
        pg_host = pg_settings.POSTGRES_HOST
        pg_port = pg_settings.POSTGRES_PORT
        pg_user = pg_settings.POSTGRES_USER
        pg_password = pg_settings.POSTGRES_PASSWORD
        pg_db = pg_settings.POSTGRES_DB
        pg_driver = pg_settings.POSTGRES_DRIVER

        pg_db_url = f"{pg_driver}://{pg_user}:{pg_password}@{pg_host}:{pg_port}/{pg_db}"
        pg_make_db_url = make_url(pg_db_url)
        logging.info(pg_db_url)

        pg_engine_db_url = URL(
            drivername=pg_driver,
            username=pg_user,
            password=pg_password,
            host=pg_host,
            port=pg_port,
            database=pg_db,
            query=pg_make_db_url.query
        )

        engine_instance = AsyncEngine(create_engine(pg_engine_db_url, echo=None, future=True, pool_pre_ping=True))

        return engine_instance
    except Exception as e:
        raise e


async def get_session_bak(engine_db=Depends(get_postgres_engine)) -> AsyncSession:
    async_session = sessionmaker(engine_db, class_=AsyncSession)
    try:
        async with async_session() as session:
            yield session
    except InterfaceError:
        logging.info("Check variable check_init_engine: ", "*" * 100)


async def get_pg_session() -> AsyncSession:
    global engine

    if not engine:
        logging.info("Start connect to database!", "*" * 100)
        for i in range(1000):
            try:
                engine = await get_postgres_engine()
                async with engine.begin() as conn:
                    logging.info("Already connect to database!")
                    break
            except Exception as e:
                engine = None
                logging.warning(f"{i} Attempting to reconnect to database... {str(e)}", "*" * 100)
                await asyncio.sleep(10)  # Use asyncio.sleep instead of time.sleep

    if not engine:
        engine = await get_postgres_engine()

    async_session = sessionmaker(engine, class_=AsyncSession)
    check_init_engine = False

    try:
        async with async_session() as session:
            yield session
    except InterfaceError:
        engine = None
        logging.info("Check variable check_init_engine: ", check_init_engine, "*" * 100)



class BaseCommand:
    def handle(self):
        raise NotImplementedError("Command must implement handle method")

    @staticmethod
    def print_usage():
        raise NotImplementedError("Command must implement print_usage method")


from typing import Union, Any

from fastapi import HTTPException
from sqlmodel import SQLModel
from sqlalchemy.ext.asyncio import AsyncSession


class CoreDelete:

    async def delete_one_record(self, db: AsyncSession, record_id: Union[int, str], model_class: SQLModel | Any):
        db_model = await db.get(model_class, record_id)
        if not db_model:
            raise HTTPException(status_code=404, detail="Record not found")

        await db.delete(db_model)
        await db.flush()
        return True



from ..base import BaseCommand
from .generate_restapi import start_create
import questionary
from rich import print as rprint
import re
import asyncio

class GenerateCommand(BaseCommand):
    def __init__(self, args):
        self.args = args
        self.options = {
            'controller': self.generate_controller,
            'service': self.generate_service,
            'restapi': self.generate_api
        }

    async def handle(self):
        rprint("[bold blue]🚀 Generate new component[/bold blue]")

        choice = questionary.select(
            "What would you like to generate?",
            choices=[
                "Controller",
                "Service",
                "RestAPI",
            ],
        ).ask()

        if not choice:
            return 1

        choice = choice.lower()

        name = questionary.text(
            f"Enter {choice} name:",
            validate=lambda text: len(text) > 0 and bool(re.match(r'^[a-zA-Z_][a-zA-Z0-9_]*$', text))
        ).ask()

        if name and choice in self.options:
            asyncio.run(self.options[choice](name))
            rprint(f"[bold green]✨ Successfully generated {choice}: {name}[/bold green]")
            return 0
        return 1

    async def generate_controller(self, name):
        # Add controller generation logic here
        print(f"Generating controller: {name}")

    async def generate_service(self, name):
        # Add service generation logic here
        print(f"Generating service: {name}")

    async def generate_api(self, name):
        await start_create()
        print(f"Generating RestAPI: {name}")

    @staticmethod
    def print_usage():
        print("  generate|gen|g    Generate new components (controller/service/restapi)")


from typing import Any
from sqlmodel import SQLModel
from sqlmodel.ext.asyncio.session import AsyncSession

from ..schemas.params import QueryParams


class CoreController:

    def __init__(self, db: AsyncSession, template_cls: SQLModel | Any):
        self.template = template_cls()
        self.db = db

    async def create_one_record(self, schema_create: SQLModel | Any):
        # Initialize the service
        service = self.template.service(self.db, self.template)

        async with self.db.begin():  # Begin the transaction
            result = await service.create_one_record(schema_create)  # Create the record

        # Accessing attributes after context safely
        await self.db.refresh(result)  # Ensure the record is refreshed after commit
        return {"result": result}

    async def update_one_record(self, record_id: int, schema_update: SQLModel | Any):
        # Initialize the service
        service = self.template.service(self.db, self.template)

        async with self.db.begin():  # Begin the transaction
            result = await service.update_one_record(record_id, schema_update)  # Update the record

        # Accessing attributes after context safely
        await self.db.refresh(result)  # Ensure the record is refreshed after commit
        return {"result": result}

    async def delete_one_record(self, record_id: int):
        # Initialize the service
        service = self.template.service(self.db, self.template)

        async with self.db.begin():  # Begin the transaction
            result = await service.delete_one_record(record_id)  # Delete the record

        return {"result": result}

    async def get_one_record(self, record_id: int):
        # Initialize the service
        service = self.template.service(self.db, self.template)
        result = await service.get_one(record_id)
        return {"result": result}

    async def get_list_record(self, param: QueryParams):
        # Initialize the service
        service = self.template.service(self.db, self.template)
        results = await service.get_list_with_count(param)
        return results

    async def get_remote_record(self, param: QueryParams):
        # Initialize the service
        service = self.template.service(self.db, self.template)
        results = await service.get_list_with_count(param)
        return results



from ..base import BaseCommand
import os

requirement = """\
fastapi~=0.115.6
uvicorn
sqlmodel~=0.0.22
sqlalchemy~=2.0.36
asyncpg
psycopg2-binary
pydantic-settings~=2.7.0
loguru~=0.7.3
setuptools~=75.6.0
wheel
pytest
aiofiles~=24.1.0
pydantic~=2.10.3
starlette~=0.41.3
"""

main="""\
from starlette.exceptions import HTTPException as StarletteHTTPException
from fastapi.exceptions import RequestValidationError
from cnb.exception.asgi import ASGIErrorLoggingMiddleware
from cnb.exception.handler import ExceptionHandlers
from cnb.middleware.log_request import AuditMiddleware
from app.controllers.register_api import router as register_router
from db.base import create_db_table
from fastapi import FastAPI
from loguru import logger
app = FastAPI()

@app.on_event("startup")
async def on_startup():
    logger.info("ASGI application startup")
    await create_db_table()
@app.on_event("shutdown")
async def shutdown_event():
    logger.info("ASGI application shutdown")
app.add_middleware(AuditMiddleware)
app.add_middleware(ASGIErrorLoggingMiddleware)
app.include_router(register_router)
# Register the handlers
handlers = ExceptionHandlers()
app.add_exception_handler(StarletteHTTPException, handlers.http_exception_handler)
app.add_exception_handler(RequestValidationError, handlers.validation_exception_handler)
"""

register_api="""\
from fastapi import APIRouter

prefix = "/api/v1"
router = APIRouter(prefix=prefix)
"""

db_base="""\
from cnb.db.postgres.session import get_postgres_engine
from loguru import logger
from app.models.register_models import *
from cnb.db.postgres.create_table import process_alter_table

global_items = globals().items()

async def create_db_table():
    logger.info(f"{'*' * 50} Start running function create_db_table {'*' * 50}")

    await process_alter_table(global_items)
    return {"message": "Created Table Successfully!"}
"""

session="""\
from cnb.db.postgres.session import get_pg_session

get_session = get_pg_session
"""

class InitCommand(BaseCommand):
    def __init__(self, args):
        self.args = args

    def handle(self):
        os.mkdir('app')
        os.mkdir('app/controllers')
        with open('app/controllers/register_api.py', 'w') as f:
            f.write(register_api)
        os.mkdir('app/models')
        os.mkdir('app/repositories')
        os.mkdir('app/schemas')
        os.mkdir('app/services')
        with open('main.py', 'w') as f:
            f.write(main)
        with open('requirements.txt', 'w') as f:
            f.write(requirement)
        os.system('pip install -r requirements.txt')
        os.mkdir('db')
        with open('db/base.py', 'w') as f:
            f.write(db_base)
        with open('db/session.py', 'w') as f:
            f.write(session)


from typing import Any
from sqlmodel import SQLModel
from sqlalchemy.ext.asyncio import AsyncSession


class CoreCreate:
    async def create_one_record(self, db: AsyncSession, model_class: SQLModel | Any, schema_create: SQLModel | Any):
        db_model = model_class.model_validate(schema_create.model_dump())
        db.add(db_model)
        await db.flush()
        return db_model



import contextvars
import logging

from loguru import logger

# Disable Uvicorn logs
# Suppress all Uvicorn logs
logging.getLogger("uvicorn").disabled = True
logging.getLogger("uvicorn.error").disabled = True
logging.getLogger("uvicorn.access").disabled = True

# Define context variables
trace_id_var = contextvars.ContextVar("trace_id", default="N/A")
span_id_var = contextvars.ContextVar("span_id", default="N/A")


def custom_formatter(record):
    trace_id = trace_id_var.get()
    span_id = span_id_var.get()
    record["extra"]["trace_id"] = trace_id
    record["extra"]["span_id"] = span_id
    return (
        "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> "
        "<level>{level}</level> "
        "{extra[trace_id]}-{extra[span_id]} "
        "<cyan>{name}.{function}</cyan> - <level>{message}</level>"
    )


# Configure Loguru with the custom formatter
logger.remove()
logger.add(
    sink=lambda msg: print(msg, end="\n"),  # Output to stdout
    format=custom_formatter,
    backtrace=True,
    diagnose=True
)


# Function to set trace and span IDs
def set_trace(trace_id, span_id):
    trace_id_var.set(trace_id)
    span_id_var.set(span_id)



import uuid

from starlette.middleware.base import BaseHTTPMiddleware
from fastapi import Request
import time
import json
from cnb.log.log_conf import set_trace, logger
from cnb.utils.sensitive import mask_sensitive_data


class AuditMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        # Start time for calculating duration
        uuid_data = str(uuid.uuid4()).replace("-", "")
        start_time = time.time()
        # Unique ID for this request
        request_id = uuid_data
        # Trace ID (passed from client or generated)
        trace_id = request.headers.get("X-Trace-ID", uuid_data)
        # New span ID for this request
        span_id = str(uuid.uuid4()).replace("-", "")

        set_trace(trace_id, span_id)

        # If part of an existing trace, get parent span
        parent_span_id = request.headers.get("X-Parent-Span-ID", "None")

        # Get user information and IP address
        user_id = request.headers.get("X-User-ID", "anonymous")
        transaction_id = request.headers.get("X-Transaction-ID", "N/A")
        user_ip = request.client.host

        extra_info = {
            'request_id': request_id,
            'trace_id': trace_id,
            'span_id': span_id,
            'parent_span_id': parent_span_id,
            'user_id': user_id,
            'ip_address': user_ip,
            'transaction_id': transaction_id,
            'method': request.method,
            'url': str(request.url),
            'status_code': "",
            "duration": "",
        }

        user_agent = request.headers.get("user-agent", "unknown")

        # Log incoming request
        logger.bind(request_id=request_id, user_id=user_id).info(
            f"Request received: {request.method} {request.url} | IP: {user_ip} | User-Agent: {user_agent}",
            **extra_info
        )

        # Mask headers
        headers = dict(request.headers)
        masked_headers = mask_sensitive_data(headers)
        logger.bind(request_id=request_id, user_id=user_id).info(f"Header: {masked_headers}")

        # Mask body if it's JSON
        body = await request.body()
        try:
            json_body = json.loads(body)
            masked_body = mask_sensitive_data(json_body)
            logger.bind(request_id=request_id, user_id=user_id).info(f"Body: {json.dumps(masked_body)}")
        except json.JSONDecodeError:
            logger.info(f"Body: [non-JSON or binary data omitted]")

        response = await call_next(request)

        process_time = time.time() - start_time

        # Log outgoing response
        logger.bind(request_id=request_id, user_id=user_id).info(
            f"Request processed: {request.method} {request.url} | Status: {response.status_code} | Time: {process_time:.3f}s"
        )

        return response



from typing import Any
from sqlalchemy.future import select
from sqlmodel import SQLModel
from sqlmodel.ext.asyncio.session import AsyncSession
from ..schemas.params import QueryParams
from cnb.crud.crud import CoreCRUD


class CoreRepository:

    def __init__(self, db: AsyncSession, template: SQLModel | Any):
        self.db = db
        self.template = template
        self.crud = CoreCRUD()

    async def create_one_record(self, schema_create: SQLModel):
        db_record = await self.crud.create_one_record(self.db, self.template.model_class, schema_create)
        return db_record

    async def update_one_record(self, record_id: int, schema_update: SQLModel):
        db_record = await self.crud.update_one_record(self.db, record_id, self.template.model_class, schema_update)
        return db_record

    async def delete_one_record(self, record_id: int):
        db_record = await self.crud.delete_one_record(self.db, record_id, self.template.model_class)
        return db_record

    async def get_one(self, db: AsyncSession, record_id: int):
        result = await db.execute(select(self.template.model_class).where(self.template.model_class.id == record_id))
        return result.scalar()

    async def get_list(self, db: AsyncSession, param: QueryParams):
        result = await self.crud.get_list(db, param, self.template.model_class)
        return result

    async def get_list_with_count(self, db: AsyncSession, param: QueryParams):
        result = await self.crud.get_list_with_count(db, param, self.template.model_class)
        return result



from pydantic import PostgresDsn, computed_field
from pydantic_core import MultiHostUrl
from pydantic_settings import BaseSettings, SettingsConfigDict


class PgSettings(BaseSettings):
    POSTGRES_HOST: str
    POSTGRES_PORT: str | int
    POSTGRES_USER: str
    POSTGRES_PASSWORD: str
    POSTGRES_DB: str
    POSTGRES_DRIVER: str

    model_config = SettingsConfigDict(
        env_file='.env',
        env_ignore_empty=True,
        extra="ignore",
    )

    def get_postgres_uri(self):
        return MultiHostUrl.build(
            scheme=self.POSTGRES_DRIVER,
            username=self.POSTGRES_USER,
            password=self.POSTGRES_PASSWORD,
            host=self.POSTGRES_HOST,
            port=self.POSTGRES_PORT,
            path=self.POSTGRES_DB
        )


pg_settings = PgSettings()



from fastapi import Request
from fastapi.responses import JSONResponse
from fastapi.exceptions import RequestValidationError
from starlette.exceptions import HTTPException as StarletteHTTPException
from loguru import logger


class ExceptionHandlers:
    @staticmethod
    async def http_exception_handler(request: Request, exc: StarletteHTTPException):
        """
        Handle HTTP exceptions.
        """
        logger.error(f"HTTP Exception: {exc.detail} | Path: {request.url.path}")
        return JSONResponse(
            {"detail": exc.detail},
            status_code=exc.status_code,
        )

    @staticmethod
    async def validation_exception_handler(request: Request, exc: RequestValidationError):
        """
        Handle validation errors.
        """
        logger.warning(f"Validation error: {exc.errors()} | Path: {request.url.path}")
        return JSONResponse(
            {"detail": exc.errors()},
            status_code=422,
        )



from starlette.middleware.base import BaseHTTPMiddleware
from loguru import logger
import traceback

from starlette.requests import Request
from starlette.responses import JSONResponse


class ASGIErrorLoggingMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        try:
            response = await call_next(request)
            return response
        except Exception as exc:
            error_details = "".join(traceback.format_exception(type(exc), exc, exc.__traceback__))
            logger.critical(
                f"ASGI Exception | Path: {request.url.path} | Method: {request.method} | Details: {error_details}"
            )
            return JSONResponse(
                {"detail": "An internal server error occurred. Please contact support."},
                status_code=500,
            )



import aiofiles
import asyncio
import shutil


async def remove_lines(file_path, lines_to_remove: list):
    # Read the file content asynchronously
    async with aiofiles.open(file_path, 'r') as file:
        lines = await file.readlines()

    # Filter out the specific lines to remove
    lines = [line for line in lines if line.strip() not in lines_to_remove]

    # Write the modified content back to the file asynchronously
    async with aiofiles.open(file_path, 'w') as file:
        await file.writelines(lines)


async def unregister_model(folder_name: str, class_name: str):
    filepath = f"app/models/register_models.py"
    # Lines to remove
    lines_to_remove = [
        f"from .{folder_name}.models import {class_name}"
    ]
    await remove_lines(filepath, lines_to_remove)


async def unregister_api(folder_name, class_name):
    filepath = f"app/controllers/register_api.py"
    # Lines to remove
    lines_to_remove = [
        f"from .v1.{folder_name}.api import router as {folder_name}_router",
        f"router.include_router({folder_name}_router, tags=[\"{class_name}\"])"
    ]
    await remove_lines(filepath, lines_to_remove)


async def unregister():
    folder_name = input("Remove folder name: ").lower().replace(" ", "_")
    class_name = input("Remove class name: ").title()
    await unregister_model(folder_name, class_name)
    await unregister_api(folder_name, class_name)
    shutil.rmtree(f"app/models/{folder_name}")
    shutil.rmtree(f"app/controllers/v1/{folder_name}")
    shutil.rmtree(f"app/services/{folder_name}")
    shutil.rmtree(f"app/repositories/{folder_name}")
    shutil.rmtree(f"app/schemas/{folder_name}")


# Run the async function
if __name__ == "__main__":
    asyncio.run(unregister())



import os
from loguru import logger


def ensure_lines_at_top(filepath, lines_to_add):
    try:
        # Read the current file content
        with open(filepath, 'r') as file:
            existing_lines = file.readlines()
    except FileNotFoundError:
        # If the file does not exist, start with an empty list
        existing_lines = []

    # Determine which lines need to be added
    lines_to_prepend = [line for line in lines_to_add if
                        line + '\n' not in existing_lines and line not in existing_lines]

    # Prepend the missing lines and write back to the file
    with open(filepath, 'w') as file:
        file.writelines([line + '\n' for line in lines_to_prepend] + existing_lines)


# Lines to add to the top of the file
lines_at_top = [
    "from starlette.exceptions import HTTPException as StarletteHTTPException",
    "from fastapi.exceptions import RequestValidationError",
    "from cnb.exception.asgi import ASGIErrorLoggingMiddleware",
    "from cnb.exception.handler import ExceptionHandlers",
    "from cnb.middleware.log_request import AuditMiddleware",
    "from app.controllers.register_api import router as register_router",
    "from db.base import create_db_table",
    "from fastapi import FastAPI",
    "from loguru import logger"
]


def ensure_lines_in_file(filepath, lines_to_add):
    try:
        # Read the existing lines in the file
        with open(filepath, 'r') as file:
            existing_lines = file.readlines()
    except FileNotFoundError:
        # If the file does not exist, start with an empty list
        existing_lines = []

    # Open the file in append mode
    with open(filepath, 'a') as file:
        for line in lines_to_add:
            # Check if the line already exists in the file
            if line + '\n' not in existing_lines and line not in existing_lines:
                # Write the line if it doesn't exist
                file.write(line + '\n')


# Lines to add to the file
lines = [
    "app = FastAPI()\n",
    "@app.on_event(\"startup\")",
    "async def on_startup():",
    "    logger.info(\"ASGI application startup\")",
    "    await create_db_table()",
    "@app.on_event(\"shutdown\")",
    "async def shutdown_event():",
    "    logger.info(\"ASGI application shutdown\")",
    "app.add_middleware(AuditMiddleware)",
    "app.add_middleware(ASGIErrorLoggingMiddleware)",
    "app.include_router(register_router)",
    "# Register the handlers",
    "handlers = ExceptionHandlers()",
    "app.add_exception_handler(StarletteHTTPException, handlers.http_exception_handler)",
    "app.add_exception_handler(RequestValidationError, handlers.validation_exception_handler)",
]


def create_file_if_not_exists(filepath: str, content_text: str = ""):
    # Ensure the directories in the path exist
    os.makedirs(os.path.dirname(filepath), exist_ok=True)

    # Check if the file exists
    if not os.path.exists(filepath):
        # Create the file
        with open(filepath, 'w') as file:
            # Create an empty file
            file.write(content_text)
        logger.info(f"File '{filepath}' created.")
    else:
        logger.info(f"File '{filepath}' already exists.")


requirements = [
    "fastapi~=0.115.6",
    "uvicorn",
    "sqlmodel~=0.0.22",
    "sqlalchemy~=2.0.36",
    "asyncpg",
    "psycopg2-binary",
    "pydantic-settings~=2.7.0",
    "loguru~=0.7.3",
    "setuptools~=75.6.0",
    "wheel",
    "pytest",
    "aiofiles~=24.1.0",
    "pydantic~=2.10.3",
    "starlette~=0.41.3"
]
db_base_file = """
from cnb.db.postgres.session import get_postgres_engine
from loguru import logger
from app.models.register_models import *
from cnb.db.postgres.create_table import process_alter_table

global_items = globals().items()

async def create_db_table():
    logger.info(f"{'*' * 50} Start running function create_db_table {'*' * 50}")

    await process_alter_table(global_items)
    return {"message": "Created Table Successfully!"}

"""
db_session_file = """
from cnb.db.postgres.session import get_pg_session

get_session = get_pg_session

"""
register_api_file = """
from fastapi import APIRouter

prefix = "/api/v1"
router = APIRouter(prefix=prefix)
"""

env_postgres_file = [
    "POSTGRES_HOST=localhost",
    "POSTGRES_PORT=5432",
    "POSTGRES_DB=postgres_db",
    "POSTGRES_USER=postgres_user",
    "POSTGRES_PASSWORD=postgres_pass",
    "POSTGRES_DRIVER=postgresql+asyncpg",
    "ENABLE_LOG_REQUEST_HEADER=True",
    "ENABLE_LOG_REQUEST_BODY=True"
]


def process():
    ensure_lines_at_top("main.py", lines_at_top)
    ensure_lines_in_file("main.py", lines)

    file_path = "app/controllers/v1/__init__.py"
    create_file_if_not_exists(file_path)

    file_path = "app/controllers/register_api.py"
    create_file_if_not_exists(file_path, register_api_file)

    file_path = "app/models/register_models.py"
    create_file_if_not_exists(file_path)

    file_path = "app/schemas/__init__.py"
    create_file_if_not_exists(file_path)

    file_path = "app/services/__init__.py"
    create_file_if_not_exists(file_path)

    file_path = "app/repositories/__init__.py"
    create_file_if_not_exists(file_path)

    file_path = "db/base.py"
    create_file_if_not_exists(file_path, db_base_file)

    file_path = "db/__init__.py"
    create_file_if_not_exists(file_path)

    file_path = "db/session.py"
    create_file_if_not_exists(file_path, db_session_file)

    file_path = "requirements.txt"
    ensure_lines_at_top(file_path, requirements)

    file_path = ".env"
    ensure_lines_at_top(file_path, env_postgres_file)


process()



import sys
from .commands.init.base import InitCommand
from .commands.generate.base import GenerateCommand

COMMANDS = {
    'init': InitCommand,

    # generate command
    'generate': GenerateCommand,
    'gen': GenerateCommand,
    'g': GenerateCommand
}

def main():
    if len(sys.argv) < 2:
        print("Usage: pyravel <command> [options]")
        print("\nAvailable commands:")
        for cmd in COMMANDS.keys():
            COMMANDS[cmd].print_usage()
        return 1

    command = sys.argv[1]
    command_args = sys.argv[2:]

    if command not in COMMANDS:
        print("Unknown command. Available commands:")
        for cmd in COMMANDS.keys():
            COMMANDS[cmd].print_usage()
        return 1

    command_class = COMMANDS[command]
    command_instance = command_class(command_args)
    return command_instance.handle()

if __name__ == "__main__":
    sys.exit(main())


from typing import Union, Any
from sqlmodel import SQLModel
from sqlalchemy import func
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.future import select

from ..core.schemas.params import QueryParams


class CoreRead:
    def get_paging(self, paging: SQLModel):
        if paging.limit <= 0:
            paging.limit = 20
        return paging

    async def get_one(self, db: AsyncSession, record_id: Union[int, str], model_class: SQLModel | Any):
        result = await db.execute(select(model_class).where(model_class.id == record_id))
        return result.scalar()

    async def get_list(self, db: AsyncSession, param: QueryParams, model_class: SQLModel | Any):
        paging = self.get_paging(param.paging)
        query = select(model_class).offset(paging.offset).limit(paging.limit)
        statement = await db.execute(query)
        result = statement.scalars().all()
        return result

    async def get_list_with_count(self, db: AsyncSession, param: QueryParams, model_class: SQLModel | Any):
        paging = self.get_paging(param.paging)
        query = select(model_class).offset(paging.offset).limit(paging.limit)
        total_count = await db.scalar(select(func.count(model_class.id)))
        statement = await db.execute(query)
        result = statement.scalars().all()
        return {"total": total_count, "result": result}



from setuptools import setup, find_packages

setup(
    name='pyravel',  # Changed name to match command
    version='0.2.1',
    author='Your Name',
    author_email='your.email@example.com',
    description='A Python package that builds files from the command line.',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'pyravel=pyravel.cli:main',
        ],
    },
    install_requires=[
        'questionary',
        'rich',
    ],
    python_requires=">=3.12",
)



import os
import asyncio
import shutil

import aiofiles
from loguru import logger


async def create_file(filepath: str, content: str):
    """
    Creates a file with the provided content if it does not already exist.
    """
    # Ensure the directory exists
    os.makedirs(os.path.dirname(filepath), exist_ok=True)

    # Check if the file already exists
    if not os.path.exists(filepath):
        # Write the content to the file asynchronously
        async with aiofiles.open(filepath, "w") as file:
            await file.write(content)
        logger.info(f"File '{filepath}' has been created successfully.")
    else:
        logger.info(f"File '{filepath}' already exists. No changes were made.")


async def create_model(folder_name: str, class_name: str):
    model_data = f"""
from sqlmodel import SQLModel, Field


class {class_name}Base(SQLModel):
    # name: str
    pass


class {class_name}({class_name}Base, table=True):
    id: int = Field(default=None, primary_key=True)
    
    """

    filepath = f"app/models/{folder_name}/models.py"
    await create_file(filepath, model_data)


async def schema_create(folder_name: str, class_name: str):
    model_data = f"""
from app.models.{folder_name}.models import {class_name}Base


class {class_name}Create({class_name}Base):
    pass

    """
    filepath = f"app/schemas/{folder_name}/create/schema.py"
    await create_file(filepath, model_data)


async def schema_update(folder_name: str, class_name: str):
    model_data = f"""
from app.schemas.{folder_name}.create.schema import {class_name}Create


class {class_name}Update({class_name}Create):
    id: int

    """

    filepath = f"app/schemas/{folder_name}/update/schema.py"
    await create_file(filepath, model_data)


async def schema_list(folder_name: str, class_name: str):
    model_data = f"""
from sqlmodel import SQLModel
from typing import List

from app.models.{folder_name}.models import {class_name}Base


class {class_name}ResponseListBase({class_name}Base):
    id: int


class {class_name}ResponseList(SQLModel):
    result: List[{class_name}ResponseListBase]
    total: int | None = 0

    """

    filepath = f"app/schemas/{folder_name}/response/list.py"
    await create_file(filepath, model_data)


async def schema_one(folder_name: str, class_name: str):
    model_data = f"""
from sqlmodel import SQLModel

from app.models.{folder_name}.models import {class_name}Base


class {class_name}ResponseOneBase({class_name}Base):
    id: int


class {class_name}ResponseOne(SQLModel):
    result: {class_name}ResponseOneBase | None

    """

    filepath = f"app/schemas/{folder_name}/response/one.py"
    await create_file(filepath, model_data)


async def schema_remote(folder_name: str, class_name: str):
    model_data = f"""
from sqlmodel import SQLModel
from typing import List

from app.models.{folder_name}.models import {class_name}Base


class {class_name}ResponseRemoteBase({class_name}Base):
    id: int

    @property
    def text(self):
        return f"{{self.id}}"


class {class_name}ResponseRemote(SQLModel):
    result: List[{class_name}ResponseRemoteBase]
    total: int | None = 0

    """

    filepath = f"app/schemas/{folder_name}/response/remote.py"
    await create_file(filepath, model_data)


async def schema_delete(folder_name: str, class_name: str):
    model_data = f"""
from sqlmodel import SQLModel


class {class_name}ResponseDelete(SQLModel):
    result: bool | None

    """

    filepath = f"app/schemas/{folder_name}/response/delete.py"
    await create_file(filepath, model_data)


async def create_service(folder_name: str, class_name: str):
    model_data = f"""
from cnb.core.services.services import CoreService


class {class_name}Service(CoreService):
    pass

    """

    filepath = f"app/services/{folder_name}/services.py"
    await create_file(filepath, model_data)


async def create_repository(folder_name: str, class_name: str):
    model_data = f"""
from cnb.core.repositories.repos import CoreRepository


class {class_name}Repository(CoreRepository):
    pass

    """

    filepath = f"app/repositories/{folder_name}/repos.py"
    await create_file(filepath, model_data)


async def create_controller(folder_name: str, class_name: str):
    model_data = f"""
from fastapi import APIRouter, Depends, HTTPException
from loguru import logger
from sqlmodel.ext.asyncio.session import AsyncSession

from app.models.{folder_name}.models import {class_name}
from app.repositories.{folder_name}.repos import {class_name}Repository
from app.schemas.{folder_name}.create.schema import {class_name}Create
from app.schemas.{folder_name}.response.delete import {class_name}ResponseDelete
from app.schemas.{folder_name}.response.list import {class_name}ResponseList
from app.schemas.{folder_name}.response.one import {class_name}ResponseOne
from app.schemas.{folder_name}.response.remote import {class_name}ResponseRemote
from app.schemas.{folder_name}.update.schema import {class_name}Update
from app.services.{folder_name}.services import {class_name}Service
from cnb.core.controllers.controllers import CoreController
from cnb.core.schemas.params import QueryParams
from db.session import get_session


class {class_name}Template:
    model_class = {class_name}
    schema_create = {class_name}Create
    schema_update = {class_name}Update
    response_one = {class_name}ResponseOne
    response_delete = {class_name}ResponseDelete
    response_list = {class_name}ResponseList
    response_remote = {class_name}ResponseRemote
    service = {class_name}Service
    repository = {class_name}Repository


router = APIRouter(prefix="/{str(folder_name).lower()}")
template_cls = {class_name}Template
controller_cls = CoreController


@router.post("/create/", response_model=template_cls.response_one)
async def create_one_record(schema_create: template_cls.schema_create, db: AsyncSession = Depends(get_session)):
    try:
        # Initialize the controller
        controller = controller_cls(db, template_cls)
        return await controller.create_one_record(schema_create)
    except Exception as e:
        logger.error(f"Error creating record: {{e}}")
        raise e


@router.post(f"/one/{{{{record_id}}}}/", response_model=template_cls.response_one)
async def get_one_record(record_id: int, db: AsyncSession = Depends(get_session)):
    try:
        # Initialize the controller
        controller = controller_cls(db, template_cls)
        result = await controller.get_one_record(record_id)
        if not result:
            raise HTTPException(status_code=404, detail="Record not found")
        return result
    except Exception as e:
        logger.error(f"Error updating record: {{e}}")
        raise e


@router.patch(f"/update/{{{{record_id}}}}/", response_model=template_cls.response_one)
async def update_one_record(record_id: int, schema_update: template_cls.schema_update,
                            db: AsyncSession = Depends(get_session)):
    try:
        # Initialize the controller
        controller = controller_cls(db, template_cls)
        return await controller.update_one_record(record_id, schema_update)
    except Exception as e:
        logger.error(f"Error updating record: {{e}}")
        raise e


@router.delete(f"/delete/{{{{record_id}}}}/", response_model=template_cls.response_delete)
async def delete_one_record(record_id: int, db: AsyncSession = Depends(get_session)):
    try:
        # Initialize the controller
        controller = controller_cls(db, template_cls)
        return await controller.delete_one_record(record_id)
    except Exception as e:
        logger.error(f"Error updating record: {{e}}")
        raise e


@router.post("/list/", response_model=template_cls.response_list)
async def get_list_record(param: QueryParams, db: AsyncSession = Depends(get_session)):
    try:
        # Initialize the controller
        controller = controller_cls(db, template_cls)
        return await controller.get_list_record(param)
    except Exception as e:
        logger.error(f"Error getting record: {{e}}")
        raise e


@router.post("/remote/", response_model=template_cls.response_remote)
async def get_remote_record(param: QueryParams, db: AsyncSession = Depends(get_session)):
    try:
        # Initialize the controller
        controller = controller_cls(db, template_cls)
        return await controller.get_list_record(param)
    except Exception as e:
        logger.error(f"Error getting record: {{e}}")
        raise e

    """

    filepath = f"app/controllers/v1/{folder_name}/api.py"
    await create_file(filepath, model_data)


async def modify_file(file_path: str, import_stm: str, router_stm: str):
    # Read the file content asynchronously
    async with aiofiles.open(file_path, 'r') as file:
        lines = await file.readlines()

    if not lines:  # Check if the file is empty
        # If empty, write both lines to the file
        async with aiofiles.open(file_path, 'w') as file:
            await file.write(import_stm)
            await file.write(router_stm)
    else:
        # Find the first empty line and insert the import statement
        for i, line in enumerate(lines):
            if line.strip() == "":
                lines.insert(i, import_stm)
                break
        else:
            # If no empty line is found, append the import statement at the end
            lines.append(import_stm)

        # Add the router inclusion at the end
        lines.append(router_stm)

        # Write the modified content back to the file asynchronously
        async with aiofiles.open(file_path, 'w') as file:
            await file.writelines(lines)


async def register_api(folder_name: str, class_name: str):
    filepath = f"app/controllers/register_api.py"
    await create_file(filepath, "")
    import_stm = f"from .v1.{folder_name}.api import router as {folder_name}_router"
    router_stm = f"router.include_router({folder_name}_router, tags=[\"{class_name}\"])\n"
    await modify_file(filepath, import_stm, router_stm)


async def register_model(folder_name: str, class_name: str):
    await create_file("app/models/register_models.py", "")
    import_stm = f"from .{folder_name}.models import {class_name}\n"
    router_stm = f""
    filepath = f"app/models/register_models.py"
    await modify_file(filepath, import_stm, router_stm)


async def start_create():
    folder_name = input("Create folder name: ").lower().replace(" ", "_")
    class_name = input("Create class name: ").title()

    await create_model(folder_name, class_name)
    await schema_create(folder_name, class_name)
    await schema_list(folder_name, class_name)
    await schema_remote(folder_name, class_name)
    await schema_one(folder_name, class_name)
    await schema_update(folder_name, class_name)
    await schema_delete(folder_name, class_name)
    await create_service(folder_name, class_name)
    await create_repository(folder_name, class_name)
    await create_controller(folder_name, class_name)
    await register_api(folder_name, class_name)
    await register_model(folder_name, class_name)


# Run the async function
if __name__ == "__main__":
    asyncio.run(start_create())



from typing import List
from loguru import logger
from sqlmodel import SQLModel, Session

from sqlalchemy import text
from fastapi import HTTPException

from cnb.db.postgres.session import get_postgres_engine

postgresql_type = {
    "DATETIME": "timestamp without time zone",
}


async def get_table_schema(db: Session, table_schema: str):
    try:
        result = await db.execute(
            text(f"""SELECT column_name, data_type FROM information_schema.columns 
            WHERE "table_schema"= :table_schema"""),
            {"table_schema": table_schema},
        )
        columns = result.fetchall()
        return dict(columns)
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


async def get_table_columns(db: Session, table_name: str, table_schema: str):
    try:
        result = await db.execute(
            text(f"""SELECT column_name, data_type FROM information_schema.columns
             WHERE "table_schema"= :table_schema AND "table_name" = :table_name"""),
            {"table_name": table_name, "table_schema": table_schema},
        )
        columns = result.fetchall()
        return dict(columns)
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))



async def process_alter_table(global_items: list) -> List[str]:
    try:
        logger.info(f"""{"*" * 50} Start running function process_alter_table {"*" * 50}""")
        alter_statement = []
        engine = await get_postgres_engine()
        async with engine.begin() as conn:
            logger.info("*" * 50, "Start Create Database Schema", "*" * 50)

            for schema in SQLModel.metadata._schemas:
                stm = f"""CREATE SCHEMA IF NOT EXISTS "{schema}";"""
                logger.info(stm)
                await conn.execute(text(stm))

            logger.info(f"""{"*" * 50} Start run function  SQLModel.metadata.create_all {"*" * 50}""")
            await conn.run_sync(SQLModel.metadata.create_all)

            logger.info("*" * 50, "Start prepare statements", "*" * 50)

            for name, obj in global_items:
                if hasattr(obj, "model_fields"):
                    schema = "public"
                    if hasattr(obj, "__table_args__"):
                        schema = obj.__table_args__.get("schema", "public")

                    table_name = obj.__tablename__
                    table_columns = await get_table_columns(conn, table_name, schema)

                    for column in obj.__table__.columns:
                        if column.name not in table_columns.keys():
                            column_type = str(column.type)
                            column_type = postgresql_type.get(column_type, column_type)
                            stm_str = f"""ALTER TABLE IF EXISTS {schema}."{table_name}" 
                            ADD COLUMN IF NOT EXISTS "{column.name}" {column_type} DEFAULT NULL;"""
                            alter_statement.append(stm_str)

            logger.info(f"""{"*" * 50} Start alter table {"*" * 50}""")
            for stmt in alter_statement:
                logger.info(stmt)
                await conn.execute(text(stmt))

    except Exception as e:
        logger.error(e)
        await conn.rollback()
        raise e
    return alter_statement
